import logging
import sys
import tempfile
from datetime import datetime
import os

from typing import TextIO, AnyStr

LOG_LEVEL = logging.ERROR
# Create a logger
logger = logging.getLogger("AgentCrew")
logger.setLevel(LOG_LEVEL)  # Set default level to DEBUG

# Create a console handler
ch = logging.StreamHandler(sys.stdout)
ch.setLevel(LOG_LEVEL)  # Set handler level

# Create a formatter and set it for the handler
formatter = logging.Formatter(
    "%(asctime)s - %(name)s - %(levelname)s - %(module)s.%(funcName)s:%(lineno)d - %(message)s"
)
ch.setFormatter(formatter)

# Add the handler to the logger
if not logger.handlers:
    logger.addHandler(ch)

# Optional: Prevent duplicate logging if this module is imported multiple times
logger.propagate = False


class FileLogIO(TextIO):
    """File-like object compatible with sys.stderr for MCP logging."""

    def __init__(self, file_format: str = "agentcrew"):
        log_dir_path = os.getenv("AGENTCREW_LOG_PATH", tempfile.gettempdir())
        os.makedirs(log_dir_path, exist_ok=True)
        self.log_path = (
            log_dir_path + f"/{file_format}_{datetime.now().timestamp()}.log"
        )
        self.file = open(self.log_path, "w+")

    def write(self, data: AnyStr) -> int:
        """Write data to the log file."""
        if isinstance(data, bytes):
            # Convert bytes to string for writing
            str_data = data.decode("utf-8", errors="replace")
        else:
            str_data = str(data)
        self.file.write(str_data)
        self.file.flush()  # Ensure data is written immediately
        return 0

    def flush(self):
        """Flush the file buffer."""
        self.file.flush()

    def close(self):
        """Close the file."""
        self.file.close()

    def fileno(self):
        """Return the file descriptor."""
        return self.file.fileno()



from typing import Any

from PySide6.QtWidgets import QApplication


class MessageEventHandler:
    """Handles message-related events in the chat UI."""

    def __init__(self, chat_window):
        from AgentCrew.modules.gui import ChatWindow

        if isinstance(chat_window, ChatWindow):
            self.chat_window = chat_window
        self.chat_window.thinking_content = ""
        self.chunk_buffer_queue = []
        self.think_buffer_queue = []

    def handle_event(self, event: str, data: Any):
        """Handle a message-related event."""
        if event == "response_chunk":
            self.handle_response_chunk(data)
        elif event == "user_message_created":
            self.handle_user_message_created(data)
        elif event == "response_completed" or event == "assistant_message_added":
            self.handle_response_completed(data)
        elif event == "thinking_started":
            self.handle_thinking_started(data)
        elif event == "thinking_chunk":
            self.handle_thinking_chunk(data)
        elif event == "thinking_completed":
            self.handle_thinking_completed()
        elif event == "user_context_request":
            self.handle_user_context_request()

    def handle_response_chunk(self, data):
        """Handle response chunks with smooth streaming."""
        chunk_text, full_response = data
        self.chunk_buffer_queue.extend(list(chunk_text))

        if full_response.strip():
            # Create bubble immediately if needed
            if (
                self.chat_window.expecting_response
                and self.chat_window.current_response_bubble is None
            ):
                self.chat_window.current_response_bubble = (
                    self.chat_window.chat_components.append_message("", False)
                )

        # Use the individual chunk for smooth streaming
        if self.chat_window.current_response_bubble:
            self.chat_window.current_response_bubble.add_streaming_chunk(
                self.chunk_buffer_queue
            )

    def handle_user_message_created(self, data):
        """Handle user message creation."""
        if self.chat_window.current_user_bubble:
            self.chat_window.current_user_bubble.message_index = (
                self.chat_window.message_handler.current_user_input_idx
            )
            self.chat_window.current_user_bubble = None
            self.chat_window.chat_scroll.verticalScrollBar().setValue(
                self.chat_window.chat_scroll.verticalScrollBar().maximum()
            )

    def handle_response_completed(self, data):
        """Handle response completion."""
        self.chunk_buffer_queue = []
        if self.chat_window.current_response_bubble:
            # Finalize streaming and ensure full content is rendered
            self.chat_window.current_response_bubble.raw_text_buffer = data
            self.chat_window.current_response_bubble.raw_text = data
            self.chat_window.current_response_bubble._finalize_streaming()
            self.chat_window.current_response_bubble.message_index = (
                len(self.chat_window.message_handler.streamline_messages) - 1
            )
        QApplication.processEvents()
        self.chat_window.chat_scroll.repaint()

    def handle_thinking_started(self, data):
        """Handle thinking process started."""
        self.think_buffer_queue = []
        agent_name = data
        self.chat_window.chat_components.add_system_message(
            f"💭 {agent_name.upper()}'s thinking process started"
        )

        # Create a new thinking bubble
        self.chat_window.current_thinking_bubble = (
            self.chat_window.chat_components.append_thinking_message("", agent_name)
        )
        self.chat_window.thinking_content = ""  # Initialize thinking content

    def handle_thinking_chunk(self, chunk):
        """Handle a chunk of the thinking process."""
        self.think_buffer_queue.extend(list(chunk))
        self.chat_window.thinking_content += chunk
        # Use smooth streaming for thinking chunks too
        if self.chat_window.current_thinking_bubble:
            self.chat_window.current_thinking_bubble.add_streaming_chunk(
                self.think_buffer_queue
            )

    def handle_thinking_completed(self):
        """Handle thinking process completion."""
        self.chat_window.display_status_message("Thinking completed.")
        # Finalize thinking stream if active
        if self.chat_window.current_thinking_bubble:
            self.chat_window.current_thinking_bubble.raw_text_buffer = (
                self.chat_window.thinking_content
            )
            self.chat_window.current_thinking_bubble._finalize_streaming()
        # Reset thinking bubble reference
        self.think_buffer_queue = []
        self.chat_window.current_thinking_bubble = None
        self.chat_window.thinking_content = ""

    def handle_user_context_request(self):
        """Handle user context request."""
        self.chat_window.chat_components.add_system_message("Refreshing my memory...")



from PySide6.QtCore import Qt
from PySide6.QtGui import QKeySequence, QShortcut
from PySide6.QtWidgets import QTextEdit
import sys


class KeyboardHandler:
    """Handles keyboard input for the chat window."""

    def __init__(self, chat_window):
        from AgentCrew.modules.gui import ChatWindow

        if isinstance(chat_window, ChatWindow):
            self.chat_window = chat_window
        # Don't setup shortcuts immediately - wait for setup_shortcuts() to be called

    def _setup_shortcuts(self):
        """Set up keyboard shortcuts."""
        # Ctrl+Enter shortcut
        self.send_shortcut = QShortcut(QKeySequence("Ctrl+Return"), self.chat_window)
        self.send_shortcut.activated.connect(self.chat_window.send_message)

        # Ctrl+Shift+C shortcut (copy last response)
        self.copy_shortcut = QShortcut(QKeySequence("Ctrl+Shift+C"), self.chat_window)
        self.copy_shortcut.activated.connect(
            self.chat_window.command_handler.copy_last_response
        )

        # Ctrl+L shortcut (clear chat)
        self.clear_shortcut = QShortcut(QKeySequence("Ctrl+L"), self.chat_window)
        self.clear_shortcut.activated.connect(
            lambda: self.chat_window.command_handler.clear_chat(requested=False)
        )

        self.history_shortcut = QShortcut(QKeySequence("Ctrl+H"), self.chat_window)
        self.history_shortcut.activated.connect(self.chat_window.toggleSidebar)

        # Escape shortcut (stop message stream) - changed from Ctrl+C to avoid conflict
        self.stop_shortcut = QShortcut(QKeySequence("Escape"), self.chat_window)
        self.stop_shortcut.activated.connect(self.chat_window.stop_message_stream)

    def _handle_completer_accept(self, completer, event):
        current_index = completer.popup().currentIndex()
        if current_index.isValid():
            completion = completer.completionModel().data(
                current_index, Qt.ItemDataRole.DisplayRole
            )
            self.chat_window.input_components.insert_completion(completion)
            completer.popup().hide()
            event.accept()
            return

    def handle_key_press(self, event):
        """Handle key press events for the message input."""
        # Handle Tab key for completion
        real_control_key = (
            Qt.KeyboardModifier.ControlModifier
            if sys.platform != "darwin"
            else Qt.KeyboardModifier.MetaModifier
        )
        if event.key() == Qt.Key.Key_Tab:
            # Select the current completion
            if self.chat_window.file_completer.popup().isVisible():
                return self._handle_completer_accept(
                    self.chat_window.file_completer, event
                )
            elif self.chat_window.command_completer.popup().isVisible():
                return self._handle_completer_accept(
                    self.chat_window.command_completer, event
                )
            QTextEdit.keyPressEvent(self.chat_window.message_input, event)

        # Handle Enter key for completion
        elif event.key() == Qt.Key.Key_Return:
            # Select the current completion
            if self.chat_window.file_completer.popup().isVisible():
                return self._handle_completer_accept(
                    self.chat_window.file_completer, event
                )
            elif self.chat_window.command_completer.popup().isVisible():
                return self._handle_completer_accept(
                    self.chat_window.command_completer, event
                )
            QTextEdit.keyPressEvent(self.chat_window.message_input, event)

        # Ctrl+Enter to send
        elif (
            event.key() == Qt.Key.Key_Return
            and event.modifiers() == Qt.KeyboardModifier.ControlModifier
        ):
            self.chat_window.send_message()
            event.accept()
            return

        # Up arrow to navigate history
        elif (
            event.key() == Qt.Key.Key_Up
            and event.modifiers() == real_control_key
            and not self.chat_window.file_completer.popup().isVisible()
        ):
            self.history_navigate(-1)
            event.accept()
            return

        # Down arrow to navigate history
        elif (
            event.key() == Qt.Key.Key_Down
            and event.modifiers() == real_control_key
            and not self.chat_window.file_completer.popup().isVisible()
        ):
            self.history_navigate(1)
            event.accept()
            return

        # Default behavior for other keys
        else:
            QTextEdit.keyPressEvent(self.chat_window.message_input, event)

    def history_navigate(self, direction):
        """Navigate through input history."""
        if not self.chat_window.message_handler.history_manager.history:
            return

        new_position = self.chat_window.history_position + direction

        if (
            0
            <= new_position
            < len(self.chat_window.message_handler.history_manager.history)
        ):
            self.chat_window.history_position = new_position
            history_entry = self.chat_window.message_handler.history_manager.history[
                self.chat_window.history_position
            ]
            self.chat_window.message_input.setText(history_entry)
        elif new_position < 0:
            self.chat_window.history_position = -1
            self.chat_window.message_input.clear()
        elif new_position >= len(
            self.chat_window.message_handler.history_manager.history
        ):
            self.chat_window.history_position = len(
                self.chat_window.message_handler.history_manager.history
            )
            self.chat_window.message_input.clear()



"""
UI effects and animations for console interface.
Handles loading animations, live displays, and other visual effects.
"""

import time
import threading
import itertools
import random
from rich.console import Console
from rich.live import Live
from rich.markdown import Markdown

from .constants import CODE_THEME
from AgentCrew.modules.chat import MessageHandler


class UIEffects:
    """Handles UI effects like loading animations and live displays."""

    def __init__(self, console: Console, message_handler: MessageHandler):
        """Initialize UI effects with a console instance."""
        self.console = console
        self.live = None
        self._live_text_data = ""
        self._loading_stop_event = None
        self._loading_thread = None
        self.message_handler = message_handler

    def _loading_animation(self, stop_event):
        """Display a loading animation in the terminal."""
        spinner = itertools.cycle(["⣾", "⣽", "⣻", "⢿", "⡿", "⣟", "⣯", "⣷"])
        fun_words = [
            "Pondering",
            "Cogitating",
            "Ruminating",
            "Contemplating",
            "Brainstorming",
            "Calculating",
            "Processing",
            "Analyzing",
            "Deciphering",
            "Meditating",
            "Daydreaming",
            "Scheming",
            "Brewing",
            "Conjuring",
            "Inventing",
            "Imagining",
        ]
        fun_word = random.choice(fun_words)

        with Live(
            "", console=self.console, auto_refresh=True, refresh_per_second=10
        ) as live:
            while not stop_event.is_set():
                live.update(f"{fun_word} {next(spinner)}")
                time.sleep(0.1)  # Control animation speed
            live.update("")  # Clear the live display when done
            live.stop()  # Stop the live display

    def start_loading_animation(self):
        """Start the loading animation."""
        if self._loading_thread and self._loading_thread.is_alive():
            return  # Already running

        self._loading_stop_event = threading.Event()
        self._loading_thread = threading.Thread(
            target=self._loading_animation, args=(self._loading_stop_event,)
        )
        self._loading_thread.daemon = True
        self._loading_thread.start()

    def stop_loading_animation(self):
        """Stop the loading animation."""
        if self._loading_stop_event:
            self._loading_stop_event.set()
        if self._loading_thread and self._loading_thread.is_alive():
            self._loading_thread.join(timeout=0.5)
        self._loading_stop_event = None
        self._loading_thread = None

    def start_streaming_response(self, agent_name: str):
        """Start streaming the assistant's response."""
        from .constants import RICH_STYLE_GREEN_BOLD
        from rich.text import Text

        self.console.print(
            Text(f"🤖 {agent_name.upper()}:", style=RICH_STYLE_GREEN_BOLD)
        )
        self.live = Live(
            "", console=self.console, refresh_per_second=24, vertical_overflow="crop"
        )
        self.live.start()

    def update_live_display(self, chunk: str):
        """Update the live display with a new chunk of the response."""
        if not self.live:
            self.start_streaming_response(self.message_handler.agent.name)

        updated_text = chunk
        self._live_text_data = updated_text

        # Only show the last part that fits in the console
        lines = updated_text.split("\n")
        height_limit = (
            self.console.size.height - 10
        )  # leave some space for other elements
        if len(lines) > height_limit:
            lines = lines[-height_limit:]

        if self.live:
            self.live.update(Markdown("\n".join(lines), code_theme=CODE_THEME))

    def finish_live_update(self):
        """Stop the live update display."""
        if self.live:
            self.console.print(self.live.get_renderable())
            self.live.update("")
            self.live.stop()
            self.live = None

    def finish_response(self, response: str):
        """Finalize and display the complete response."""
        if self.live:
            self.live.update("")
            self.live.stop()
            self.live = None

        # Replace \n with two spaces followed by \n for proper Markdown line breaks
        markdown_formatted_response = response.replace("\n", "  \n")
        self.console.print(Markdown(markdown_formatted_response, code_theme=CODE_THEME))

    def cleanup(self):
        """Clean up all running effects."""
        self.stop_loading_animation()
        self.finish_live_update()
